package RDF::Source::DBI::Statement;
use strict;
use CGI::Carp;
use Data::Dumper;

sub init
{
    my( $self ) = @_;
    $self->{'source'}{'dbh'} = $self->model->dbh;
}


sub dbh
{
    my( $self ) = @_;
    return $self->{'source'}{'dbh'};
}

sub init_by_id
{
    my( $self, $id ) = @_;

    my $sql = qq(select id, pred, subj, obj, fact
		 from statements
		 where id=?
		 );

#	print join("", "\n[[\t statement \t init_by_id \t", $id );
		 
    $self->init_by_sql( $sql, $id );
}

sub init_by_uri
{
    my( $self, $uri ) = @_;

    die "No uri specified" unless $uri;

    my $nslocal = $self->model->ns_local;

    $uri =~ /^$nslocal(\d+)$/ or confess "$uri is not a valid uri for a statement";
    my $id = $1;

    my $sql = qq(select id, pred, subj, obj, fact
		 from statements
		 where id=?
		 );

#	print join("", "\n[[\t statement \t init_by_uri \t", $uri );

    $self->init_by_sql( $sql, $id );
}

sub init_by_sql
{
    my( $self, $sql, @values ) = @_;

    my $sth = $self->dbh->prepare_cached( $sql )
	or confess("($sql) ".$self->dbh->errstr);
    $sth->execute( @values );
    my( $id, $pred_id, $subj_id, $obj_id, $fact ) = $sth->fetchrow_array;
    $sth->finish;

    not defined $id and confess "Can't find the resource";
      
    $self->{'simple'}{'pred'}    = $self->model->get_node_by_id( $pred_id );
    $self->{'simple'}{'subj'}    = $self->model->get_node_by_id( $subj_id );
    $self->{'simple'}{'obj'}     = $self->model->get_node_by_id( $obj_id );
    $self->{'simple'}{'fact'}    = $fact;
    $self->{'simple'}{'uri'}     = $self->model->ns_local.$id;

    $self->{'source'}{'id'}       = $id;

    # Ouh! Do not trust these. They do not get updated!
#    $self->{'source'}{'pred_id'}  = $pred_id;
#    $self->{'source'}{'subj_id'}  = $subj_id;
#    $self->{'source'}{'obj_id'}   = $obj_id;

}


sub delete
{
    my( $self ) = @_;

    my $sth = $self->dbh->prepare_cached(qq(delete from statements
					    where id = ?
					    ));
    $sth->execute( $self->id )
	or die $sth->errstr;

    undef $self;
}

sub store
{
    my( $self ) = @_;

    my $id = $self->id                          or die "statement id not specified";

    my $pred = $self->pred->id                  or die "statement pred not specified";
    my $subj = $self->subj->id                  or die "statement subj not specified";
    my $obj  = $self->obj->id                   or die "statement obj not specified";
    my $fact = $self->fact                      or die "statment fact not specified";

    my $sth = $self->dbh->prepare_cached(qq(update statements set
					    pred = ?,
					    subj = ?,
					    obj = ?,
					    fact = ?
					    where id = ?
					    ));

#	print join("", "\n[[\t statement \t store \t", $id );
    
    $sth->execute( $pred, $subj, $obj, $fact, $id )
	or die $self->dbh->errstr;
}



sub id      { $_[0]->{'source'}{'id'} }
sub pred_id { $_[0]->{'source'}{'pred_id'} }
sub subj_id { $_[0]->{'source'}{'subj_id'} }
sub obj_id  { $_[0]->{'source'}{'obj_id'} }



1;




